#  Based on: http://www.djangosnippets.org/snippets/73/
#
#  Modified by Sean Reifschneider to be smarter about surrounding page
#  link context.  For usage documentation see:
#
#     http://www.tummy.com/Community/Articles/django-pagination/

from django import template

register = template.Library()


def paginator(context, adjacent_pages=2):
    """
    To be used in conjunction with the object_list generic view.

    Adds pagination context variables for use in displaying first, adjacent and
    last page links in addition to those created by the object_list generic
    view.

    """
    paginator = context['paginator']
    page = context['deploys'].number
    startPage = max(page - adjacent_pages, 1)
    if startPage <= 3:
        startPage = 1
    endPage = page + adjacent_pages + 1
    if endPage >= paginator.num_pages - 1:
        endPage = paginator.num_pages + 1
    page_numbers = [n for n in range(startPage, endPage)
                    if n > 0 and n <= paginator.num_pages]

    return {
        'paginator': paginator,
        'page': page,
        'pages': paginator.num_pages,
        'page_numbers': page_numbers,
        'next': context['deploys'].next_page_number,
        'previous': context['deploys'].previous_page_number,
        'has_next': context['deploys'].has_next,
        'has_previous': context['deploys'].has_previous,
        'show_first': 1 not in page_numbers,
        'show_last': paginator.num_pages not in page_numbers,
    }

register.inclusion_tag('deploys/paginator.html', takes_context=True)(paginator)



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.teams.views import List


class ListTeamViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @patch('requests.get')
    def test_should_use_list_template(self, get):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = []
        get.return_value = response_mock

        response = List.as_view()(self.request)

        self.assertIn("teams/list.html", response.template_name)
        expected = []
        self.assertListEqual(expected, response.context_data['teams'])
        get.assert_called_with(
            '{}/teams'.format(settings.TSURU_HOST),
            headers={'authorization': 'admin'})

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_return_empty_list_when_status_is_204(self, token_is_valid, get):
        token_is_valid.return_value = True
        get.return_value = Mock(status_code=204)

        response = List.as_view()(self.request)

        self.assertIn("teams/list.html", response.template_name)
        expected = []
        self.assertListEqual(expected, response.context_data['teams'])
        get.assert_called_with(
            '{}/teams'.format(settings.TSURU_HOST),
            headers={'authorization': 'admin'})



from django.test import TestCase
from django.core.urlresolvers import reverse

import mock
import requests


class HealthcheckTest(TestCase):

    @mock.patch("requests.get")
    def test_healthcheck_ok(self, get_mock):
        get_mock.return_value = mock.Mock(status_code=200, text="WORKING")

        response = self.client.get(reverse("healthcheck"))

        self.assertEqual(200, response.status_code)
        self.assertEqual("WORKING", response.content)

    @mock.patch("requests.get")
    def test_healthcheck_connection_error(self, get_mock):
        get_mock.side_effect = requests.exceptions.ConnectionError()

        response = self.client.get(reverse("healthcheck"))

        self.assertEqual(500, response.status_code)
        self.assertEqual("Failed to connect to tsuru.", response.content)

    @mock.patch("requests.get")
    def test_healthcheck_wrong_status_code(self, get_mock):
        get_mock.return_value = mock.Mock(status_code=500, text="WORKING")

        response = self.client.get(reverse("healthcheck"))

        self.assertEqual(500, response.status_code)
        self.assertEqual("Failed to connect to tsuru.", response.content)

    @mock.patch("requests.get")
    def test_healthcheck_wrong_text(self, get_mock):
        get_mock.return_value = mock.Mock(status_code=200, text="not expected")

        response = self.client.get(reverse("healthcheck"))

        self.assertEqual(500, response.status_code)
        self.assertEqual("Failed to connect to tsuru.", response.content)



from mock import patch

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

import httpretty
import json

from tsuru_dashboard import settings
from tsuru_dashboard.admin.views import NodeInfo


class NodeInfoViewTest(TestCase):
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def setUp(self, token_is_valid):
        token_is_valid.return_value = True
        httpretty.enable()

        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {'tsuru_token': 'tokentest'}

        self.address = 'http://127.0.0.2:4243'

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        self.containers = [
            {"id": "blabla", "type": "python", "appname": "myapp", "hostaddr": "http://127.0.0.2:4243"}
        ]
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.containers),
            status=200
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        self.nodes = {
            "nodes": [
                {
                    "Status": "ready",
                    "Metadata": {
                        "pool": "tsuru1",
                        "iaas": "ec2",
                        "LastSuccess": "2015-11-16T18:44:36-02:00",
                    },
                    "Address": "http://127.0.0.2:4243"
                },
                {
                    "Status": "ready",
                    "Metadata": {
                        "pool": "tsuru2",
                        "iaas": "ec2",
                        "LastSuccess": "2015-11-16T18:44:36-02:00",
                    },
                    "Address": "http://127.0.0.3:4243"
                },
            ]
        }
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.nodes),
            status=200
        )
        self.response = NodeInfo.as_view()(self.request, address=self.address)

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_should_use_list_template(self):
        self.assertIn("admin/node_info.html", self.response.template_name)

    def teste_should_get_list_of_containers_from_tsuru(self):
        self.assertListEqual(self.containers, self.response.context_data["containers"])

    def teste_should_get_node_info_from_tsuru(self):
        self.assertDictEqual(self.nodes["nodes"][0], self.response.context_data["node"])

    def test_get_request_run_url_should_not_return_404(self):
        response = self.client.get(reverse('node-info', args=[self.address.replace("http://", "")]))
        self.assertNotEqual(404, response.status_code)

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def teste_should_get_list_of_containers_empty_list(self, token_is_valid):
        httpretty.reset()
        token_is_valid.return_value = True

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        httpretty.register_uri(
            httpretty.GET,
            url,
            status=204
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.nodes),
            status=200
        )
        response = NodeInfo.as_view()(self.request, address=self.address)
        self.assertListEqual([], response.context_data["containers"])

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def teste_should_get_list_of_containers_on_error(self, token_is_valid):
        httpretty.reset()
        token_is_valid.return_value = True

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        httpretty.register_uri(
            httpretty.GET,
            url,
            status=500
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.nodes),
            status=200
        )
        response = NodeInfo.as_view()(self.request, address=self.address)
        self.assertListEqual([], response.context_data["containers"])

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def teste_should_get_empty_node(self, token_is_valid):
        httpretty.reset()
        token_is_valid.return_value = True

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.containers),
            status=200
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        httpretty.register_uri(
            httpretty.GET,
            url,
            status=204
        )
        response = NodeInfo.as_view()(self.request, address=self.address)
        self.assertFalse(response.context_data["node"])



from django.test import TestCase
from django.forms import EmailField
from django.forms.widgets import PasswordInput

from tsuru_dashboard.auth import forms


class LoginFormTest(TestCase):
    def test_login_should_have_username_and_password_fields(self):
        self.assertIn('username', forms.LoginForm.base_fields)
        self.assertIn('password', forms.LoginForm.base_fields)

    def test_widget_for_password_field_should_be_password(self):
        field = forms.LoginForm.base_fields['password']
        self.assertIsInstance(field.widget, PasswordInput)

    def test_username_should_have_at_most_60_characters(self):
        field = forms.LoginForm.base_fields['username']
        self.assertEqual(60, field.max_length)

    def test_password_should_have_at_least_6_characters(self):
        field = forms.LoginForm.base_fields['password']
        self.assertEqual(6, field.min_length)

    def test_username_field_should_accept_only_email_values(self):
        field = forms.LoginForm.base_fields['username']
        self.assertIsInstance(field, EmailField)



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.services.views import ListService


class ListServiceViewTest(TestCase):
    @patch('requests.get')
    def test_should_use_list_template(self, get):
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        expected = {"teste": "teste"}

        response_mock = Mock(status_code=200)
        response_mock.json.return_value = expected
        get.return_value = response_mock

        response = ListService.as_view()(request)

        self.assertIn("services/list.html", response.template_name)
        self.assertDictEqual(expected, response.context_data['services'])

        url = '{}/services/instances'.format(settings.TSURU_HOST)
        get.assert_called_with(url, headers={'authorization': 'admin'})



from django.test import TestCase
from django.template import Context, Template


class TestFilter(TestCase):

    def test_datetime_filter(self):
        html = '{% load filters_deploys %}'
        html += '{{ timestamp|string_to_date }}'
        template = Template(html)
        context = Context({'timestamp': '2014-02-27T17:55:34.2-03:00'})
        timestamp = "Feb. 27, 2014, 5:55 p.m."
        self.assertEqual(timestamp, template.render(context))

    def test_time_to_string_filter(self):
        html = '{% load filters_deploys %}'
        html += '{{ duration|time_to_string }}'
        template = Template(html)
        context = Context({'duration': '203902854662'})
        self.assertEqual("03m23s", template.render(context))



from django.http import HttpResponseRedirect
from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard.auth.views import Logout


class TestLogoutWhenUserIsLoggedIn(TestCase):
    def setUp(self):
        self.request = RequestFactory().get('/logout/')
        self.request.session = {'tsuru_token': 'my beautiful token'}
        self.response = Logout.as_view()(self.request)

    def test_should_remove_tsuru_token_from_session(self):
        token = self.request.session.get('tsuru_token')
        self.assertIsNone(token)

    def test_should_redirect_to_index(self):
        self.assertIsInstance(self.response, HttpResponseRedirect)
        self.assertEqual('/', self.response['Location'])


class TestLogoutWhenUserIsLogout(TestCase):
    def setUp(self):
        self.request = RequestFactory().get('/logout/')
        self.request.session = {}
        self.response = Logout.as_view()(self.request)

    def test_should_redirect_to_index(self):
        self.assertIsInstance(self.response, HttpResponseRedirect)
        self.assertEqual('/', self.response['Location'])



from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.metrics.backend import ElasticSearch, get_backend, MetricNotEnabled, NET_AGGREGATION
from tsuru_dashboard.metrics import views

from mock import patch, Mock
import json
import datetime


class MetricViewTest(TestCase):
    def request(self):
        request = RequestFactory().get("/ble/?metric=cpu_max")
        request.session = {"tsuru_token": "token"}
        return request

    @patch("requests.get")
    def test_get_app(self, get_mock):
        response_mock = Mock()
        response_mock.json.return_value = {}
        get_mock.return_value = response_mock

        view = views.Metric()
        view.request = self.request()
        app = view.get_app("app_name")

        self.assertDictEqual(app, {})
        url = "{}/apps/app_name".format(settings.TSURU_HOST)
        headers = {"authorization": "token"}
        get_mock.assert_called_with(url, headers=headers)

    @patch("requests.get")
    def test_get_envs(self, get_mock):
        env_mock = [{"name": "VAR", "value": "value"}]
        response_mock = Mock()
        response_mock.json.return_value = env_mock
        get_mock.return_value = response_mock

        view = views.Metric()
        view.request = self.request()
        envs = view.get_envs(self.request(), "app_name")

        self.assertDictEqual(envs, {"VAR": "value"})
        url = "{}/apps/app_name/env".format(settings.TSURU_HOST)
        headers = {"authorization": "token"}
        get_mock.assert_called_with(url, headers=headers)

    @patch("tsuru_dashboard.metrics.backend.get_backend")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get(self, token_mock, get_backend_mock):
        token_mock.return_value = True
        backend_mock = Mock()
        backend_mock.cpu_max.return_value = {}
        get_backend_mock.return_value = backend_mock

        v = views.Metric

        original_get_app = v.get_app
        v.get_app = Mock()
        v.get_app.return_value = {}

        original_get_envs = v.get_envs
        v.get_envs = Mock()
        v.get_envs.return_value = {}
        view = v.as_view()

        def cleanup():
            v.get_app = original_get_app
            v.get_envs = original_get_envs

        self.addCleanup(cleanup)

        request = RequestFactory().get("/ble/?metric=cpu_max&date_range=2h/h&interval=30m&process_name=web")
        request.session = {"tsuru_token": "token"}

        response = view(request, app_name="app_name")

        self.assertEqual(response.status_code, 200)
        get_backend_mock.assert_called_with({'envs': {}}, 'token', date_range=u'2h/h', process_name=u'web')
        backend_mock.cpu_max.assert_called_with(interval=u'30m')

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_bad_request(self, token_mock):
        request = RequestFactory().get("")
        request.session = {"tsuru_token": "token"}
        token_mock.return_value = True

        v = views.Metric
        view = v.as_view()

        response = view(request, app_name="app_name")

        self.assertEqual(response.status_code, 400)


class GetBackendTest(TestCase):
    @patch("requests.get")
    def test_envs_from_api(self, get_mock):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {
            "METRICS_BACKEND": "logstash",
            "METRICS_ELASTICSEARCH_HOST": "http://easearch.com",
            "METRICS_LOGSTASH_HOST": "logstash.com"
        }
        get_mock.return_value = response_mock

        app = {"name": "appname", "units": [{"ProcessName": "web"}]}

        backend = get_backend(app, 'token')
        self.assertIsInstance(backend, ElasticSearch)

    @patch("requests.get")
    def test_envs_from_app(self, get_mock):
        get_mock.return_value = Mock(status_code=500)

        app = {"name": "appname", "envs": {"ELASTICSEARCH_HOST": "ble"}, "units": [{"ID": "id"}]}

        backend = get_backend(app, 'token')
        self.assertIsInstance(backend, ElasticSearch)

    @patch("requests.get")
    def test_without_metrics(self, get_mock):
        get_mock.return_value = Mock(status_code=404)
        app = {"name": "appname"}

        with self.assertRaises(MetricNotEnabled):
            get_backend(app, 'token')


class ElasticSearchTest(TestCase):
    def setUp(self):
        self.maxDiff = None
        self.es = ElasticSearch("http://url.com", "index", "app")
        self.index = ".measure-tsuru-{}".format(datetime.datetime.utcnow().strftime("%Y.%m.%d"))

    @patch("requests.post")
    def test_cpu_max(self, post_mock):
        self.es.process = Mock()
        self.es.cpu_max()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "cpu_max")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query()))

    @patch("requests.post")
    def test_mem_max(self, post_mock):
        self.es.process = Mock()
        self.es.mem_max()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "mem_max")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query()))

    @patch("requests.post")
    def test_swap(self, post_mock):
        self.es.process = Mock()
        self.es.swap()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "swap")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query()))

    @patch("requests.post")
    def test_units(self, post_mock):
        self.es.units()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "cpu_max")
        aggregation = {"units": {"cardinality": {"field": "host"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_requests_min(self, post_mock):
        self.es.requests_min()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {"sum": {"sum": {"field": "count"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_response_time(self, post_mock):
        self.es.response_time()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {
            "stats": {"stats": {"field": "value"}},
            "percentiles": {"percentiles": {"field": "value"}}
        }
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_http_methods(self, post_mock):
        self.es.http_methods()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {"method": {"terms": {"field": "method"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_status_code(self, post_mock):
        self.es.status_code()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "response_time")
        aggregation = {"status_code": {"terms": {"field": "status_code"}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_connections(self, post_mock):
        self.es.connections()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "connection")
        script = "doc['connection.raw'].value + doc['connection'].value"
        aggregation = {"connection": {"terms": {"script": script}}}
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=aggregation)))

    @patch("requests.post")
    def test_netrx(self, post_mock):
        self.es.netrx()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "netrx")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=NET_AGGREGATION)))

    @patch("requests.post")
    def test_nettx(self, post_mock):
        self.es.nettx()
        url = "{}/{}/{}/_search".format(self.es.url, self.index, "nettx")
        post_mock.assert_called_with(url, data=json.dumps(self.es.query(aggregation=NET_AGGREGATION)))

    def test_process(self):
        data = {
            "took": 86,
            "timed_out": False,
            "_shards": {
                "total": 266,
                "successful": 266,
                "failed": 0
            },
            "hits": {
                "total": 644073,
                "max_score": 0,
                "hits": []
            },
            "aggregations": {
                "date": {
                    "buckets": [
                        {
                            "key_as_string": "2015-07-21T19:35:00.000Z",
                            "key": 1437507300000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        },
                        {
                            "key_as_string": "2015-07-21T19:36:00.000Z",
                            "key": 1437507360000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        }
                    ]
                }
            }
        }
        expected = {
            "data": {
                "max": [[1437507300000, 97517568], [1437507360000, 97517568]],
                "min": [[1437507300000, 97517568], [1437507360000, 97517568]],
                "avg": [[1437507300000, 97517568], [1437507360000, 97517568]],
            },
            "min": 97517568,
            "max": 97517569
        }
        d = self.es.process(data)
        self.assertDictEqual(d, expected)

    def test_process_custom_formatter(self):
        data = {
            "took": 86,
            "timed_out": False,
            "_shards": {
                "total": 266,
                "successful": 266,
                "failed": 0
            },
            "hits": {
                "total": 644073,
                "max_score": 0,
                "hits": []
            },
            "aggregations": {
                "date": {
                    "buckets": [
                        {
                            "key_as_string": "2015-07-21T19:35:00.000Z",
                            "key": 1437507300000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        },
                        {
                            "key_as_string": "2015-07-21T19:36:00.000Z",
                            "key": 1437507360000,
                            "doc_count": 9,
                            "stats": {
                                "min": 97517568,
                                "max": 97517568,
                                "avg": 97517568
                            }
                        }
                    ]
                }
            }
        }
        expected = {
            "data": {
                "max": [[1437507300000, 93], [1437507360000, 93]],
                "min": [[1437507300000, 93], [1437507360000, 93]],
                "avg": [[1437507300000, 93], [1437507360000, 93]],
            },
            "min": 93,
            "max": 94
        }
        d = self.es.process(data, formatter=lambda x: x / (1024 * 1024))
        self.assertDictEqual(d, expected)

    def test_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.process(data, formatter=lambda x: x / (1024 * 1024))
        self.assertDictEqual(d, expected)

    def test_unit_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.base_process(data, self.es.units_process)
        self.assertDictEqual(d, expected)

    def test_request_min_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.base_process(data, self.es.requests_min_process)
        self.assertDictEqual(d, expected)

    def test_connections_process_no_aggregation(self):
        data = {
            "took": 1,
            "timed_out": False,
            "_shards": {
                "total": 0,
                "successful": 0,
                "failed": 0
            },
            "hits": {
                "total": 0,
                "max_score": 0,
                "hits": []
            }
        }
        expected = {'data': {}, 'max': 1, 'min': 0}
        d = self.es.base_process(data, self.es.connections_process)
        self.assertDictEqual(d, expected)



from django.test import TestCase
from django.test.client import RequestFactory

import mock
import httpretty

import json

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import Callback


class CallbackViewTest(TestCase):

    @httpretty.activate
    @mock.patch("requests.post")
    def test_callback(self, post_mock):
        url = "{}/docker/healing".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=403)

        response_mock = mock.Mock(status_code=200)
        response_mock.json.return_value = {"token": "xpto"}

        post_mock.return_value = response_mock

        request = RequestFactory().get('/', {"code": "somecode"})
        request.META['HTTP_HOST'] = 'localhost:3333'
        request.session = {"next_url": "/apps"}

        response = Callback.as_view()(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/apps")

        self.assertEqual(request.session["tsuru_token"], "type xpto")
        self.assertDictEqual(request.session["permissions"], {"healing": False})

        expected_url = 'http://localhost:8080/auth/login'
        expected_data = json.dumps({
            "redirectUrl": "http://localhost:3333/auth/callback/",
            "code": "somecode"
        })

        post_mock.assert_called_with(expected_url, data=expected_data)

    @mock.patch("requests.post")
    def test_callback_wrong_status_code(self, post_mock):
        request = RequestFactory().get('/')
        request.META['HTTP_HOST'] = 'localhost:3333'

        response = Callback.as_view()(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/auth/login")

    @httpretty.activate
    @mock.patch("requests.post")
    def test_callback_with_healing_permission(self, post_mock):
        url = "{}/docker/healing".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status_code=200)

        response_mock = mock.Mock(status_code=200)
        response_mock.json.return_value = {"token": "xpto"}

        post_mock.return_value = response_mock

        request = RequestFactory().get('/', {"code": "somecode"})
        request.META['HTTP_HOST'] = 'localhost:3333'
        request.session = {"next_url": "/apps"}

        response = Callback.as_view()(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/apps")

        self.assertEqual(request.session["tsuru_token"], "type xpto")
        self.assertDictEqual(request.session["permissions"], {"healing": True})

        expected_url = 'http://localhost:8080/auth/login'
        expected_data = json.dumps({
            "redirectUrl": "http://localhost:3333/auth/callback/",
            "code": "somecode"
        })

        post_mock.assert_called_with(expected_url, data=expected_data)



from django.test import TestCase
from django.forms import CharField

from tsuru_dashboard.services import forms


class ServiceFormTest(TestCase):
    def test_forms_should_contains_ServiceForm(self):
        self.assertTrue(hasattr(forms, 'ServiceForm'))

    def test_should_have_name_field(self):
        self.assertIn('name', forms.ServiceForm.base_fields)

    def test_name_field_should_have_CharField(self):
        field = forms.ServiceForm.base_fields['name']
        self.assertIsInstance(field, CharField)

    def test_name_field_should_have_at_most_50_characters(self):
        field = forms.ServiceForm.base_fields['name']
        self.assertEqual(50, field.max_length)
